"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Core = void 0;
const trigger_1 = require("../trigger");
const CoreState_1 = require("./CoreState");
const Logger_1 = __importDefault(require("./Logger"));
class Core {
    constructor(dataBase) {
        this._endpoints = {};
        this._models = {};
        this._services = [];
        this.dataBase = dataBase;
        this._coreState = CoreState_1.CoreState.Destroy;
        this.trigger = new trigger_1.Trigger();
    }
    createAndStart() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.create().then(() => this.start());
        });
    }
    stopAndDestroy() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.stop().then(() => this.destroy());
        });
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.info("Create Core");
            if (this._coreState !== CoreState_1.CoreState.Destroy) {
                Logger_1.default.error("Bad currently state. It must be destroy.");
                return;
            }
            this._coreState = CoreState_1.CoreState.Create;
            // Create connection to database
            yield this.dataBase.connect();
            Logger_1.default.info("Init Models");
            for (const name in this._models) {
                Logger_1.default.info(`Init Model "${name}"`);
                yield this._models[name].init(this);
                yield this.dataBase.createTable(this._models[name]);
            }
            // Create endpoints
            Logger_1.default.info("Create Endpoints");
            for (const name in this._endpoints) {
                Logger_1.default.info(`Init Endpoint "${name}"`);
                yield this._endpoints[name].init(this);
                Logger_1.default.info(`Create Endpoint "${name}"`);
                yield this._endpoints[name].create();
            }
            Logger_1.default.info("Init Services");
            for (const event of this._services) {
                yield event.init(this);
            }
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.info("Start Core");
            if (this._coreState !== CoreState_1.CoreState.Create) {
                Logger_1.default.error("Bad currently state. It must be create.");
                return;
            }
            this._coreState = CoreState_1.CoreState.Start;
            Logger_1.default.info("Start Endpoints");
            for (const name in this._endpoints) {
                Logger_1.default.info(`Start Endpoint "${name}"`);
                yield this._endpoints[name].start();
            }
            Logger_1.default.info("Start Services");
            for (const event of this._services) {
                yield event.start();
            }
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.info("Stop Core");
            if (this._coreState !== CoreState_1.CoreState.Start) {
                Logger_1.default.error("Bad currently state. It must be start.");
                return;
            }
            this._coreState = CoreState_1.CoreState.Stop;
            Logger_1.default.info("Stop Endpoints");
            for (const name in this._endpoints) {
                Logger_1.default.info(`Stop Endpoint "${name}"`);
                yield this._endpoints[name].stop();
            }
            Logger_1.default.info("Stop Services");
            for (const event of this._services) {
                yield event.stop();
            }
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.info("Destroy Core");
            if (this._coreState !== CoreState_1.CoreState.Stop) {
                Logger_1.default.error("Bad currently state. It must be stop.");
                return;
            }
            this._coreState = CoreState_1.CoreState.Destroy;
            for (const name in this._endpoints) {
                Logger_1.default.info(`Destroy Endpoint "${name}"`);
                this._endpoints[name].destroy();
            }
            Logger_1.default.info("Disconnect MongoDB");
            yield this.dataBase.disconnect();
            this.trigger.clear();
        });
    }
    addEndpoint(name, endpoint) {
        Logger_1.default.info(`Add new Endpoint "${name}"`);
        if (this._coreState !== CoreState_1.CoreState.Destroy) {
            Logger_1.default.error("Bad currently state. It must be destroy.");
            return;
        }
        this._endpoints[name] = endpoint;
    }
    addModel(init) {
        const model = init(this);
        Logger_1.default.info(`Add new Model "${model.name}"`);
        if (this._coreState !== CoreState_1.CoreState.Destroy) {
            Logger_1.default.error("Bad currently state. It must be create.");
            return;
        }
        this._models[model.name] = model;
    }
    addService(service) {
        Logger_1.default.info("Add new Service");
        if (this._coreState !== CoreState_1.CoreState.Destroy) {
            Logger_1.default.error("Bad currently state. It must be destroy.");
            return;
        }
        this._services.push(service);
    }
    removeService(service) {
        Logger_1.default.info("Remove a Service");
        if (this._coreState !== CoreState_1.CoreState.Destroy) {
            Logger_1.default.error("Bad currently state. It must be destroy.");
            return;
        }
        this._services = this._services.filter(item => item !== service);
    }
    addTrigger(listener) {
        this.trigger.addListener(listener);
    }
    getEndpoint(name) {
        return this._endpoints[name];
    }
    getModel(name) {
        return this._models[name];
    }
    getState() {
        return this._coreState;
    }
    getModels() {
        const models = [];
        for (const name in this._models) {
            models.push(this._models[name]);
        }
        return models;
    }
}
exports.Core = Core;
//# sourceMappingURL=Core.js.map