"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("./index");
const mongodb_1 = require("mongodb");
class MongoDB extends index_1.DataBase {
    constructor(port, name) {
        super();
        this.collection = {};
        this.port = port;
        this.name = name;
    }
    static buildToMongoDBFilter(filter) {
        const dbFilter = {};
        for (const key in filter) {
            const data = filter[key];
            switch (data.type) {
                case "Id":
                    if (Array.isArray(data.value)) {
                        const ids = [];
                        for (const id of data.value) {
                            ids.push(new mongodb_1.ObjectID(id));
                        }
                        dbFilter[key] = {
                            $in: ids
                        };
                    }
                    else {
                        dbFilter[key] = new mongodb_1.ObjectID(data.value);
                    }
                    break;
                case "Refs":
                    if (Array.isArray(data.value)) {
                        const ids = [];
                        for (const id of data.value) {
                            ids.push(id);
                        }
                        dbFilter[key] = {
                            $in: ids
                        };
                    }
                    else {
                        dbFilter[key] = data.value;
                    }
                case "Boolean":
                case "String":
                    dbFilter[key] = data.value;
                    break;
                case "Number":
                    if (data.value) {
                        dbFilter[key] = data.value;
                    }
                    else {
                        const filterObject = {};
                        if (data.min) {
                            filterObject.$gte = data.min;
                        }
                        if (data.max) {
                            filterObject.$lte = data.max;
                        }
                        dbFilter[key] = filterObject;
                    }
                    break;
            }
        }
        return dbFilter;
    }
    static buildToMongoDBFindOption(options) {
        const dbOptions = {};
        if (options.limit) {
            dbOptions.limit = options.limit;
        }
        if (options.offset) {
            dbOptions.skip = options.offset;
        }
        return dbOptions;
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            return mongodb_1.connect(`mongodb://localhost:${this.port}/${this.name}`, {
                useNewUrlParser: true,
                useUnifiedTopology: true
            }).then(client => {
                this._mongoDBClient = client;
                this._database = client.db(this.name);
            });
        });
    }
    disconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._mongoDBClient.isConnected()) {
                return this._mongoDBClient.close();
            }
        });
    }
    createTable(model) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._database.createCollection(model.name).then((collection) => {
                this.collection[model.name] = collection;
            });
        });
    }
    deleteByModel(model, filter) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.collection[model.name].deleteMany(MongoDB.buildToMongoDBFilter(filter)).then();
        });
    }
    findByModel(model, filter, option = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.collection[model.name]
                .find(MongoDB.buildToMongoDBFilter(filter), MongoDB.buildToMongoDBFindOption(option))
                .toArray();
        });
    }
    insertByModel(data, model) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.collection[model.name].insertOne(data).then((result) => {
                return result.ops[0];
            });
        });
    }
    updateByModel(data, model, filter) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.collection[model.name]
                .updateMany(MongoDB.buildToMongoDBFilter(filter), { $set: data }).then();
        });
    }
}
exports.default = MongoDB;
//# sourceMappingURL=MongoDB.js.map