"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("./index");
class VirtualDB extends index_1.DataBase {
    constructor() {
        super(...arguments);
        this.tables = [];
    }
    getTableByName(name) {
        for (const table of this.tables) {
            if (table.name === name) {
                return table;
            }
        }
        return null;
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
        });
    }
    createTable(model) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getTableByName(model.name) === null) {
                this.tables.push(new VirtualTable(model.name));
            }
            else {
                throw new Error("Table exist");
            }
        });
    }
    disconnect() {
        return __awaiter(this, void 0, void 0, function* () {
        });
    }
    clear() {
        this.tables = [];
    }
    findByModel(model, filter, option) {
        return __awaiter(this, void 0, void 0, function* () {
            const table = this.getTableByName(model.name);
            if (table === null) {
                throw new Error("Table not exist");
            }
            else {
                return table.find(filter, option);
            }
        });
    }
    insertByModel(data, model) {
        return __awaiter(this, void 0, void 0, function* () {
            const table = this.getTableByName(model.name);
            if (table === null) {
                throw new Error("Table exist");
            }
            else {
                return table.insert(data);
            }
        });
    }
    updateByModel(data, model, filter) {
        return __awaiter(this, void 0, void 0, function* () {
            const table = this.getTableByName(model.name);
            if (table === null) {
                throw new Error("Table exist");
            }
            else {
                return table.update(data, filter);
            }
        });
    }
    deleteByModel(model, filter) {
        return __awaiter(this, void 0, void 0, function* () {
            const table = this.getTableByName(model.name);
            if (table === null) {
                throw new Error("Table exist");
            }
            else {
                return table.delete(filter);
            }
        });
    }
}
exports.default = VirtualDB;
class VirtualTable {
    constructor(name) {
        this.data = [];
        this.name = name;
    }
    insert(data) {
        return __awaiter(this, void 0, void 0, function* () {
            this.data.push(data);
            return data;
        });
    }
    update(data, filter) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const d of this.data) {
                if (this.filter(d, filter)) {
                    for (const key in data) {
                        d[key] = data[key];
                    }
                }
            }
            return;
        });
    }
    delete(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i = 0; i < this.data.length; i++) {
                if (this.filter(this.data[i], filter)) {
                    this.data[i] = null;
                }
            }
            this.data = this.data.filter((d) => {
                return d !== null;
            });
            return;
        });
    }
    find(filter, option) {
        return __awaiter(this, void 0, void 0, function* () {
            const founds = [];
            let index = 0;
            let offset = 0;
            let limit = -1;
            // Set limits
            if (option) {
                if (option.offset)
                    offset = option.offset;
                if (option.limit)
                    limit = option.limit;
            }
            // Search data
            for (const d of this.data) {
                if (this.filter(d, filter)) {
                    index++;
                    if (index <= offset) {
                        founds.push(d);
                    }
                    if (0 < limit && limit <= index) {
                        break;
                    }
                }
            }
            return founds;
        });
    }
    filter(value, filter) {
        let bad = false;
        found: for (const filterKey in filter) {
            const options = filter[filterKey];
            const data = value[filterKey];
            switch (options.type) {
                case "Number":
                    if (options.min) {
                        if (data < options.min) {
                            bad = true;
                            break found;
                        }
                    }
                    if (options.max) {
                        if (data > options.max) {
                            bad = true;
                            break found;
                        }
                    }
                case "String":
                case "Boolean":
                    if (options.value) {
                        if (data !== options.value) {
                            bad = true;
                            break found;
                        }
                    }
                    break;
                case "Id":
                case "Refs":
                    if (Array.isArray(options.value)) {
                        let found = false;
                        for (const v of options.value) {
                            if (v === data) {
                                found = true;
                                break;
                            }
                        }
                        if (!found) {
                            bad = true;
                            break found;
                        }
                    }
                    else {
                        if (data !== options.value) {
                            bad = true;
                            break found;
                        }
                    }
                    break;
            }
        }
        return false;
    }
}
//# sourceMappingURL=VirtualDB.js.map