"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ExecuteMiddleware_1 = __importDefault(require("./middleware/ExecuteMiddleware"));
const PermissionUtils_1 = require("./permission/PermissionUtils");
function generator(option) {
    const endpoints = [];
    const permission = PermissionUtils_1.buildPermission(option.permission || []);
    // Read all paths
    for (const path of option.paths) {
        endpoints.push(...buildPath(option.core, permission, path, '', []));
    }
    return endpoints;
}
exports.default = generator;
function buildPath(core, permission, option, rootPath, rootMiddleware) {
    // Overwrite old settings
    const path = `${rootPath}/${option.path}`;
    const middleware = [...rootMiddleware];
    const endpoints = [];
    const permissionBuild = permission.get(option.permission);
    // Exits new middleware
    if (option.middleware) {
        if (Array.isArray(option.middleware)) {
            middleware.push(...option.middleware);
        }
        else {
            middleware.push(option.middleware);
        }
    }
    // It is a execute function
    if (option.execute) {
        endpoints.push(buildApiRequest(core, path, middleware, option.execute, permissionBuild));
    }
    // Go to next upper layer
    if (option.children && Array.isArray(option.children)) {
        for (const child of option.children) {
            endpoints.push(...buildPath(core, permission, child, path, middleware));
        }
    }
    return endpoints;
}
function buildApiRequest(core, path, middleware, execute, permissionBuild) {
    let permission;
    if (permissionBuild) {
        permission = permissionBuild.execute;
    }
    return {
        path,
        execute: (core, args) => __awaiter(this, void 0, void 0, function* () {
            const executeArgs = {
                meta: args.meta ? args.meta : {},
                data: args.data ? args.data : {}
            };
            // Execute the pre middleware
            return ExecuteMiddleware_1.default(core, middleware, executeArgs, "pre").then((result) => {
                // Exist error goto directly into the past middleware
                if (result.error && result.error.length > 0) {
                    return result;
                }
                // Exist a permission function
                if (permission) {
                    // Execute permission
                    return permission(core, executeArgs.meta).then((permissionResult) => {
                        // Exist a error object
                        if (typeof permissionResult !== "boolean") {
                            return { error: permissionResult };
                        }
                        // If have all permission than execute
                        return execute(core, executeArgs);
                    });
                }
                // Execute without a permission
                return execute(core, executeArgs);
            }).then((fromResult) => {
                // Execute the past middleware
                return ExecuteMiddleware_1.default(core, middleware, executeArgs, "post").then((result) => {
                    return fromResult;
                });
            });
        })
    };
}
//# sourceMappingURL=Generator.js.map