"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.argumentBuilding = void 0;
const BuildFilter_1 = __importDefault(require("../utils/BuildFilter"));
function argumentBuilding(args, options) {
    return __awaiter(this, void 0, void 0, function* () {
        let executeList = Promise.resolve({ arguments: args, error: [] });
        let optionList = {};
        for (const option of options) {
            optionList = Object.assign(optionList, option);
            executeList = executeList.then((result) => {
                if (result.error.length > 0) {
                    return result;
                }
                return argumentBuildingAtOnePriority(result.arguments, option);
            });
        }
        return executeList.then((result) => {
            if (result.error.length > 0) {
                return result;
            }
            return {
                arguments: disableHiddenArguments(result.arguments, optionList),
                error: []
            };
        });
    });
}
exports.argumentBuilding = argumentBuilding;
function disableHiddenArguments(args, options) {
    const newArguments = {};
    for (const key in args) {
        if (!options[key].hidden) {
            newArguments[key] = args[key];
        }
    }
    return newArguments;
}
function argumentBuildingAtOnePriority(args, options) {
    return __awaiter(this, void 0, void 0, function* () {
        // TODO Use old arguments
        const requireError = checkArguments(args, options);
        if (requireError.length > 0) {
            return {
                arguments: {},
                error: requireError
            };
        }
        const newArgs = [];
        for (const argument in args) {
            if (options[argument]) {
                newArgs.push(executeArgument(argument, args[argument], options[argument]));
            }
            else {
                newArgs.push(Promise.resolve({ name: argument, data: args[argument], error: [] }));
            }
        }
        return Promise.all(newArgs).then((results) => {
            const buildArgs = {};
            let errors = [];
            for (const result of results) {
                if (result.error.length > 0) {
                    errors = errors.concat(result.error);
                }
                else {
                    buildArgs[result.name] = result.data;
                }
            }
            if (errors.length > 0) {
                return {
                    arguments: {},
                    error: errors
                };
            }
            return {
                arguments: buildArgs,
                error: []
            };
        });
    });
}
function checkArguments(args, options) {
    const errors = [];
    for (const option in options) {
        if (options[option].require && args[option] === undefined) {
            errors.push({
                name: "Argument require ",
                type: "Argument",
                message: `The Argument '${option}' is require.`
            });
        }
    }
    return errors;
}
function executeArgument(name, args, option) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!option) {
            return {
                name,
                data: args[name],
                error: [{
                        name: "No defined Argument",
                        type: "Argument",
                        message: `The argument '${name}' is not defined.`
                    }]
            };
        }
        if (option.model) {
            return executeReferenceArgument(name, args, option);
        }
        return {
            name: option.name,
            data: args[name],
            error: [{
                    name: "No defined Argument",
                    type: "Argument",
                    message: `The argument '${name}' is not defined.`
                }]
        };
    });
}
function executeReferenceArgument(name, args, { select, model, filter }) {
    return __awaiter(this, void 0, void 0, function* () {
        const subFilter = {};
        for (const key in filter) {
            subFilter[key] = Object.assign(filter[key], {
                priority: 0,
                hidden: false
            });
        }
        return argumentBuilding(args, [subFilter]).then((result) => {
            if (Array.isArray(result)) {
                return {
                    name,
                    data: args[name],
                    error: result
                };
            }
            return model.find(BuildFilter_1.default(args)).then((values) => {
                const resultObject = [];
                for (const value of values) {
                    resultObject.push(value[select]);
                }
                return {
                    name,
                    data: resultObject,
                    error: []
                };
            });
        });
    });
}
//# sourceMappingURL=ArgumentBuilding.js.map