"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildToApiRequestArguments = exports.updateSelector = void 0;
function updateSelector(rootFilter, filter, core, model, changeData) {
    const defaultFilter = {
        auto: rootFilter.auto,
        local: {},
        global: rootFilter.global
    };
    if (filter) {
        if (filter.auto !== undefined) {
            defaultFilter.auto = filter.auto;
        }
        defaultFilter.local = generateAutoArguments(core.getModel(model), changeData);
        for (const arg in filter.global) {
            defaultFilter.global[arg] = buildFilterArguments(filter.global[arg], core);
        }
        for (const arg in filter.local) {
            defaultFilter.local[arg] = buildFilterArguments(filter.local[arg], core);
        }
    }
    return defaultFilter;
}
exports.updateSelector = updateSelector;
function buildFilterArguments(data, core) {
    if (typeof data === "string" || typeof data === "boolean" || data.name) {
        return data;
    }
    const refData = data;
    const schema = core.getModel(refData.model).schema;
    if (refData.filter) {
        for (const arg in refData.filter) {
            refData.filter[arg] = buildFilterArguments(refData.filter[arg], core);
        }
    }
    else {
        refData.filter = {};
    }
    if (refData.autoArguments) {
        const autoFilter = {};
        for (const key in schema) {
            if (!schema[key].protection) {
                autoFilter[key] = true;
            }
        }
        refData.filter = Object.assign(autoFilter, refData.filter);
    }
    return refData;
}
function generateAutoArguments(model, changeData) {
    const filter = {};
    for (const key in model.schema) {
        if (!model.schema[key].protection) {
            if (!(changeData && model.schema[key].type === "Id")) {
                filter[key] = {
                    name: key,
                    require: false
                };
            }
        }
    }
    return filter;
}
function optimizeArguments(filter, core, hidden) {
    const args = {};
    if (filter) {
        for (const key in filter) {
            const value = filter[key];
            if (typeof value === "boolean") {
                if (value) {
                    args[key] = {
                        hidden,
                        priority: 0,
                        name: key,
                        require: false
                    };
                }
            }
            else if (typeof value === "string") {
                args[key] = {
                    hidden,
                    priority: 0,
                    name: value,
                    require: false
                };
            }
            else if (typeof value === "object") {
                let priority = 0;
                if (value.priority) {
                    priority = value.priority;
                }
                if (value.name) {
                    args[key] = {
                        hidden: value.hidden,
                        priority: value.priority,
                        name: value.name,
                        require: value.require ? value.require : false
                    };
                }
                else {
                    args[key] = {
                        hidden: value.hidden,
                        priority: value.priority,
                        model: core.getModel(value.model),
                        select: value.select,
                        require: value.require ? value.require : false,
                        filter: optimizeArguments(value.filter, core, false)
                    };
                }
            }
        }
    }
    return args;
}
function sortPriority(filters) {
    const sortedObject = new Map();
    for (const filter of filters) {
        for (const key in filter) {
            const priority = filter[key].priority;
            if (!sortedObject.has(priority)) {
                sortedObject.set(priority, {});
            }
            sortedObject.get(priority)[key] = filter[key];
        }
    }
    const sortedArray = [];
    while (sortedObject.size > 0) {
        let currentHeightPriority = Number.MIN_VALUE;
        for (const priority of sortedObject.keys()) {
            if (priority > currentHeightPriority) {
                currentHeightPriority = priority;
            }
        }
        sortedArray.push(sortedObject.get(currentHeightPriority));
        sortedObject.delete(currentHeightPriority);
    }
    return sortedArray;
}
function buildToApiRequestArguments(filter, core) {
    return sortPriority([optimizeArguments(filter.local, core, false), optimizeArguments(filter.global, core, true)]);
}
exports.buildToApiRequestArguments = buildToApiRequestArguments;
//# sourceMappingURL=SelectorArguments.js.map