"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DescriptionRequest = exports.DescriptionGroup = void 0;
const PermissionUtils_1 = require("../permission/PermissionUtils");
class DescriptionGroup {
    constructor(path, name, details = "") {
        this.children = [];
        this.requests = [];
        this.path = path;
        this.name = name;
        this.details = details;
    }
}
exports.DescriptionGroup = DescriptionGroup;
class DescriptionRequest {
    constructor(path, name, details) {
        this.param = [];
        this.paramsOptional = [];
        this.success = [];
        this.error = [];
        this.path = path;
        this.name = name;
        this.details = details;
    }
}
exports.DescriptionRequest = DescriptionRequest;
function generator(name, url, option) {
    const mainGroup = new DescriptionGroup(url, name);
    const permission = PermissionUtils_1.buildPermission(option.permission || []);
    // Read all paths
    for (const path of option.paths) {
        buildPath(mainGroup, "", permission, path);
    }
    return mainGroup;
}
exports.default = generator;
function buildPath(root, rootPath, permission, option, extendsDescription = []) {
    // Overwrite old settings
    let description = [];
    const path = `${rootPath}/${option.path}`;
    const permissionBuild = permission.get(option.permission);
    // Exist a description
    if (option.descriptor) {
        description = description.concat(option.descriptor);
    }
    // Exits new middleware
    if (option.middleware) {
        const middleware = Array.isArray(option.middleware) ? option.middleware : [option.middleware];
        for (const m of middleware) {
            extendsDescription = extendsDescription.concat(m.description());
        }
    }
    if (permissionBuild && permissionBuild.description) {
        description = description.concat(permissionBuild.description);
    }
    description = description.concat(extendsDescription);
    let request = null;
    if (option.execute) {
        // Search meta
        for (const d of description) {
            if (d.object === "meta") {
                request = new DescriptionRequest(path, d.name, d.details);
            }
        }
        // Found a correct meta description of a api request
        if (request !== null) {
            for (const d of description) {
                switch (d.object) {
                    // Read params
                    case "param":
                        const param = {
                            type: d.type,
                            meta: d.metaData,
                            description: d.description,
                            name: d.name
                        };
                        if (d.require)
                            request.param.push(param);
                        else
                            request.paramsOptional.push(param);
                        break;
                    // Read results
                    case "result":
                        const result = {
                            type: d.type,
                            details: d.details,
                            message: d.message
                        };
                        if (d.success)
                            request.success.push(result);
                        else
                            request.error.push(result);
                        break;
                }
            }
        }
    }
    let group = root;
    for (const d of description) {
        if (d.object === "group") {
            const subGroup = new DescriptionGroup(path, d.name, d.details);
            group.children.push(subGroup);
            group = subGroup;
        }
    }
    if (request !== null) {
        group.requests.push(request);
    }
    // Go to next upper layer
    if (option.children && Array.isArray(option.children)) {
        for (const child of option.children) {
            buildPath(group, path, permission, child, extendsDescription);
        }
    }
}
//# sourceMappingURL=DocBuilder.js.map