"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const DocBuilder_1 = __importDefault(require("./DocBuilder"));
function default_1(name, url, option) {
    const docs = DocBuilder_1.default(name, url, option);
    return `${createHeader(docs)}\n\n${createNavigation(docs)}\n\n${createGroup(docs)}`;
}
exports.default = default_1;
function createHeader(docs) {
    return `# ${docs.name}\n\n\`\`${docs.path}\`\``;
}
function createNavigation(docs, sub = "") {
    let text = "<ul>\n";
    let index = 1;
    for (const group of docs.children) {
        const counter = sub.length === 0 ? `${index}.` : sub.length === 2 ? sub + `${index}` : sub + `.${index}`;
        text += "<li>\n";
        text += `${counter}  ${group.name} ${createNavigation(group, counter)}`;
        text += "</li>\n";
        index++;
    }
    return text + "</ul>\n";
}
function createGroup(docs, sub = "") {
    let text = "";
    for (const request of docs.requests) {
        text += `## ${request.name}\n\n`;
        text += `Path: \`\`${request.path}\`\`\n\n`;
        text += `> ${request.details}\n\n`;
        if (request.param.length > 0) {
            text += ` **Param** \n\n`;
            text += createParamList(request.param);
        }
        if (request.paramsOptional.length > 0) {
            text += ` **Optional Param** \n\n`;
            text += createParamList(request.paramsOptional);
        }
        if (request.success.length > 0) {
            text += ` **Success** \n\n`;
            text += createSuccessList(request.success);
        }
        if (request.error.length > 0) {
            text += ` **Error** \n\n`;
            text += createErrorList(request.error);
        }
    }
    let index = 1;
    for (const group of docs.children) {
        const counter = sub.length === 0 ? `${index}.` : sub.length === 2 ? sub + `${index}` : sub + `.${index}`;
        text += `# ${counter} ${group.name}\n\n`;
        text += `> ${group.details}\n\n`;
        text += `${createGroup(group, counter)}\n\n`;
        index++;
    }
    return text + "";
}
function createParamList(params) {
    let text = "| Name | Type | Description | As Metadata|\n";
    text += "| --- | --- | --- | ---|\n";
    for (const param of params) {
        text += `| ${param.name} | ${param.type}  | ${param.description}  | ${param.meta ? 'X' : ''} |\n`;
    }
    return text + "\n\n";
}
function createSuccessList(results) {
    let text = "<ul>";
    for (const result of results) {
        text += "<li>\n";
        text += ` <b>${result.message}</b>\n\n `;
        if (result.details)
            text += `\`\`${result.details}\`\`\n\n `;
        text += "</li>\n";
    }
    return text + "</ul>\n\n";
}
function createErrorList(results) {
    let text = "<ul>\n";
    for (const result of results) {
        text += "<li>\n";
        text += ` <b>${result.message}</b>\n\n `;
        if (result.details)
            text += `\`\`${result.details}\`\`\n\n `;
        text += "</li>\n";
    }
    return text + "</ul>\n\n";
}
//# sourceMappingURL=MDGenerator.js.map