"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildToApiRequestArguments = exports.updateFilter = void 0;
function updateFilter(rootFilter, filter, core, model) {
    const defaultFilter = {
        autoArguments: rootFilter.autoArguments,
        localArguments: {},
        globalArguments: rootFilter.globalArguments
    };
    if (filter) {
        if (filter.autoArguments !== undefined) {
            defaultFilter.autoArguments = filter.autoArguments;
        }
        defaultFilter.localArguments = generateAutoArguments(core.getModel(model));
        for (const arg in filter.globalArguments) {
            defaultFilter.globalArguments[arg] = buildFilterArguments(filter.globalArguments[arg], core);
        }
        for (const arg in filter.localArguments) {
            defaultFilter.localArguments[arg] = buildFilterArguments(filter.localArguments[arg], core);
        }
    }
    return defaultFilter;
}
exports.updateFilter = updateFilter;
function buildFilterArguments(data, core) {
    if (typeof data === "string" || typeof data === "boolean" || data.name) {
        return data;
    }
    const refData = data;
    const schema = core.getModel(refData.model).schema;
    if (refData.filter) {
        for (const arg in refData.filter) {
            refData.filter[arg] = buildFilterArguments(refData.filter[arg], core);
        }
    }
    else {
        refData.filter = {};
    }
    if (refData.autoArguments) {
        const autoFilter = {};
        for (const key in schema) {
            if (!schema[key].protection) {
                autoFilter[key] = true;
            }
        }
        refData.filter = Object.assign(autoFilter, refData.filter);
    }
    return refData;
}
function generateAutoArguments(model) {
    const filter = {};
    for (const key in model.schema) {
        if (!model.schema[key].protection) {
            filter[key] = {
                name: key,
                require: false
            };
        }
    }
    return filter;
}
function optimizeArguments(filter, core) {
    const args = {};
    if (filter) {
        for (const key in filter) {
            const value = filter[key];
            if (typeof value === "boolean") {
                if (value) {
                    args[key] = {
                        priority: 0,
                        name: key,
                        require: false
                    };
                }
            }
            else if (typeof value === "string") {
                args[key] = {
                    priority: 0,
                    name: value,
                    require: false
                };
            }
            else if (typeof value === "object") {
                let priority = 0;
                if (value.priority) {
                    priority = value.priority;
                }
                if (value.name) {
                    args[key] = {
                        priority,
                        name: value.name,
                        require: value.require ? value.require : false
                    };
                }
                else {
                    args[key] = {
                        priority,
                        model: core.getModel(value.model),
                        select: value.select,
                        require: value.require ? value.require : false,
                        filter: optimizeArguments(value.filter, core)
                    };
                }
            }
        }
    }
    return args;
}
function buildToApiRequestArguments(filter, core) {
    return Object.assign(optimizeArguments(filter.localArguments, core), optimizeArguments(filter.globalArguments, core));
}
exports.buildToApiRequestArguments = buildToApiRequestArguments;
//# sourceMappingURL=Filter.js.map