"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildPermission = void 0;
const core_1 = require("../../../core");
function buildPermission(permission) {
    // Create map
    const map = new Map();
    // Generate all execute function
    for (const per of permission) {
        build(map, per);
    }
    return map;
}
exports.buildPermission = buildPermission;
function build(map, permission, root = [], descriptor = []) {
    // Complete the permission call
    root.push(permission);
    if (permission.descriptor) {
        descriptor = descriptor.concat(permission.descriptor);
    }
    // Generate execute permission
    map.set(permission.name, { execute: createExecuteFunction(root), description: descriptor });
    // Exist children
    if (permission.children) {
        // Copy root permission
        const roots = root.map((v) => v);
        const descriptions = descriptor.map((v) => v);
        // Goto into the children permission
        for (const child of permission.children) {
            build(map, child, [...roots], [...descriptions]);
        }
    }
}
function createExecuteFunction(permission) {
    return function (core, metaData) {
        return __awaiter(this, void 0, void 0, function* () {
            let trigger = Promise.resolve(true);
            for (const { name, has } of permission) {
                trigger = trigger.then((from) => {
                    if (typeof from !== "boolean" && from.length > 0) {
                        return from;
                    }
                    return has(core, metaData).then((result) => {
                        if (typeof result === "boolean") {
                            if (result)
                                return true;
                            return [{
                                    type: core_1.ErrorType.Permission,
                                    message: `The permission '${name}' stop the request.`
                                }];
                        }
                        else {
                            return result;
                        }
                    });
                });
            }
            return trigger;
        });
    };
}
//# sourceMappingURL=PermissionUtils.js.map