"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
function buildFilter(filter) {
    const object = {};
    if (filter && typeof filter === "object") {
        for (const key in filter) {
            const value = filter[key];
            if (typeof value === "number" || typeof value === "string" || Array.isArray(value)) {
                object[key] = value;
            }
            if (typeof value === "object") {
                const numberObject = {};
                if (value.min && typeof value.min === "number") {
                    numberObject.min = value.min;
                }
                if (value.max && typeof value.max === "number") {
                    numberObject.max = value.max;
                }
                object[key] = numberObject;
            }
        }
    }
    return object;
}
function buildFindOption(filter) {
    const object = {};
    if (filter && typeof filter === "object") {
        if (filter.limit && typeof filter.limit === "number") {
            object.limit = filter.limit;
        }
        if (filter.offset && typeof filter.offset === "number") {
            object.offset = filter.offset;
        }
    }
    return object;
}
function buildDataValue(filter) {
    const object = {};
    if (filter && typeof filter === "object") {
        for (const key in filter) {
            const value = filter[key];
            if (typeof value === "number" || typeof value === "string" || value === null) {
                object[key] = value;
            }
        }
    }
    return object;
}
function executeMiddleware(middleware, args) {
    let promise = new Promise((resolve) => {
        resolve({
            middleware: [],
            validation: [],
            data: [],
        });
    });
    for (const ware of middleware) {
        promise = promise.then((result) => {
            if (result.middleware.length > 0) {
                return result;
            }
            return ware.call(args).then((errors) => {
                result.middleware.push(...errors);
                return result;
            });
        });
    }
    return promise;
}
function buildApiRequest(model, settings) {
    const endpoints = [];
    if (settings.read) {
        endpoints.push({
            path: `${settings.path}`,
            type: "Read",
            execute: (args = {}) => __awaiter(this, void 0, void 0, function* () {
                return executeMiddleware(settings.middleware, args).then((result) => {
                    if (result.middleware.length > 0) {
                        return result;
                    }
                    return model.find(buildFilter(args.filter), buildFindOption(args.option)).then((data) => {
                        for (const d of data) {
                            result.data.push(model.toExport(d));
                        }
                        return result;
                    });
                });
            })
        });
    }
    if (settings.insert) {
        endpoints.push({
            path: `${settings.path}/insert`,
            type: "Insert",
            execute: (args = {}) => __awaiter(this, void 0, void 0, function* () {
                return executeMiddleware(settings.middleware, args).then((result) => {
                    if (result.middleware.length > 0) {
                        return result;
                    }
                    return model.insert(buildDataValue(args.data)).then((data) => {
                        if (Array.isArray(data)) {
                            result.validation.push(...data);
                        }
                        else {
                            result.data.push(model.toExport(data));
                        }
                        return result;
                    });
                });
            })
        });
    }
    if (settings.update) {
        endpoints.push({
            path: `${settings.path}/update`,
            type: "Update",
            execute: (args = {}) => __awaiter(this, void 0, void 0, function* () {
                return executeMiddleware(settings.middleware, args).then((result) => {
                    if (result.middleware.length > 0) {
                        return result;
                    }
                    return model.update(buildDataValue(args.data), buildFilter(args.filter)).then((error) => {
                        result.validation.push(...error);
                        return result;
                    });
                });
            })
        });
    }
    if (settings.delete) {
        endpoints.push({
            path: `${settings.path}/delete`,
            type: "Delete",
            execute: (args = {}) => __awaiter(this, void 0, void 0, function* () {
                return executeMiddleware(settings.middleware, args).then((result) => {
                    if (result.middleware.length > 0) {
                        return result;
                    }
                    return model.delete(buildFilter(args.filter)).then((error) => {
                        return result;
                    });
                });
            })
        });
    }
    return endpoints;
}
function buildApiRequests(option, settings) {
    let endpointSettings = settings;
    if (!endpointSettings) {
        endpointSettings = {
            middleware: [],
            path: "",
            read: false,
            insert: false,
            update: false,
            delete: false
        };
    }
    endpointSettings.path += `/${option.path}`;
    if (option.middleware) {
        endpointSettings.middleware.push(option.middleware);
    }
    if (option.read) {
        endpointSettings.read = option.read;
    }
    if (option.insert) {
        endpointSettings.insert = option.insert;
    }
    if (option.update) {
        endpointSettings.update = option.update;
    }
    if (option.delete) {
        endpointSettings.delete = option.delete;
    }
    const endpoints = buildApiRequest(option.model, endpointSettings);
    if (option.children && Array.isArray(option.children)) {
        for (const child of option.children) {
            endpoints.push(...buildApiRequests(child, endpointSettings));
        }
    }
    return endpoints;
}
function ApiRequestGenerator(option) {
    const endpoints = [];
    for (const path of option.paths) {
        endpoints.push(...buildApiRequests(path));
    }
    return endpoints;
}
exports.default = ApiRequestGenerator;
//# sourceMappingURL=ApiRequestGenerator.js.map