import { Middleware, MiddlewareError } from "../middleware/Middleware";
import { Model } from "../../model/Model";
import { DataValue } from "../../model/data/DataValue";
import { ValidatorError } from "../../model/data/Validator";
export declare type RequestSettings = {
    middleware: Middleware[];
    path: string;
    read: boolean;
    insert: boolean;
    update: boolean;
    delete: boolean;
};
export declare type RequestOption = {
    paths?: PathOption[];
};
export declare type PathOption = {
    path: string;
    middleware?: Middleware;
    model?: Model;
    read?: boolean;
    insert?: boolean;
    update?: boolean;
    delete?: boolean;
    children?: PathOption[];
};
export declare type Request = {
    path: string;
    type: "Read" | "Insert" | "Update" | "Delete";
    execute: RequestFunction;
};
export declare type RequestArgument = {
    [key: string]: any;
};
export declare type RequestResult = EndpointError & {
    data?: DataValue[];
};
export declare type EndpointError = {
    middleware: MiddlewareError[];
    validation: ValidatorError[];
};
export declare type RequestFunction = (args?: RequestArgument) => Promise<RequestResult>;
