"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ReadModel_1 = __importDefault(require("./functions/ReadModel"));
const InsertModel_1 = __importDefault(require("./functions/InsertModel"));
const UpdateModel_1 = __importDefault(require("./functions/UpdateModel"));
const DeleteModel_1 = __importDefault(require("./functions/DeleteModel"));
const ExecuteFunction_1 = __importDefault(require("./functions/ExecuteFunction"));
function buildApiRequest(core, settings) {
    const endpoints = [];
    const { path, middleware, execute, model } = settings;
    // It is a execute function
    if (execute) {
        endpoints.push(ExecuteFunction_1.default(path, core, execute, middleware));
        // Or a default model request
    }
    else if (model && typeof model === "string") {
        // Read model from core
        const coreModel = core.getModel(model);
        // Add default function
        if (settings.read) {
            endpoints.push(ReadModel_1.default(path, coreModel, middleware));
        }
        if (settings.insert) {
            endpoints.push(InsertModel_1.default(path, coreModel, middleware));
        }
        if (settings.update) {
            endpoints.push(UpdateModel_1.default(path, coreModel, middleware));
        }
        if (settings.delete) {
            endpoints.push(DeleteModel_1.default(path, coreModel, middleware));
        }
    }
    return endpoints;
}
// The main layer setting
const DefaultSettings = {
    middleware: [],
    path: "",
    execute: null,
    model: null,
    read: false,
    insert: false,
    update: false,
    delete: false
};
function buildApiRequests(core, option, settings = Object.assign({}, DefaultSettings)) {
    // Overwrite old settings
    settings.path += `/${option.path}`;
    // Exits new middleware
    if (option.middleware) {
        settings.middleware.push(option.middleware);
    }
    // It is a execute function
    if (option.execute) {
        settings.execute = option.execute;
        // Or a default model request
    }
    else if (option.model) {
        settings.model = option.model;
        settings.read = option.read ? option.read : false;
        settings.insert = option.insert ? option.insert : false;
        settings.update = option.update ? option.update : false;
        settings.delete = option.delete ? option.delete : false;
    }
    // Build the endpoint for this path layer
    const endpoints = buildApiRequest(core, settings);
    // Go to next upper layer
    if (option.children && Array.isArray(option.children)) {
        for (const child of option.children) {
            endpoints.push(...buildApiRequests(core, child, Object.assign({}, settings)));
        }
    }
    return endpoints;
}
function apiRequestGenerator(option) {
    const endpoints = [];
    // Read all paths
    for (const path of option.paths) {
        endpoints.push(...buildApiRequests(option.core, path));
    }
    return endpoints;
}
exports.default = apiRequestGenerator;
//# sourceMappingURL=ApiRequestGenerator.js.map