"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestApi = void 0;
const express_1 = __importDefault(require("express"));
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const cors_1 = __importDefault(require("cors"));
const body_parser_1 = __importDefault(require("body-parser"));
const __1 = require("../..");
const Endpoint_1 = require("../Endpoint");
class RestApi extends Endpoint_1.Endpoint {
    constructor(options = {}) {
        super();
        this.DefaultPort = 4000;
        this._app = express_1.default();
        this._app.use(body_parser_1.default.json());
        this._app.use(cors_1.default());
        this.options = options;
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.options.https) {
                this._server = https_1.default.createServer({
                    key: this.options.key,
                    cert: this.options.cert
                }, this._app);
            }
            else {
                this._server = http_1.default.createServer(this._app);
            }
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const port = this.options.port | this.DefaultPort;
            __1.Logger.info(`RestApi: Start server on ${port}`);
            this._server.listen(port);
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._server) {
                this._server.close();
            }
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            this._server = null;
        });
    }
    useRequest(request) {
        __1.Logger.info(`RestApi: Add request at '${request.path}'`);
        this._app.post(request.path, (req, res) => {
            const args = req.body;
            __1.Logger.info(`RestApi: Request at path '${request.path}'`);
            request.execute(this.core, args).then((result) => {
                res.send(JSON.stringify(result));
            });
        });
    }
}
exports.RestApi = RestApi;
//# sourceMappingURL=RestApi.js.map