"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestEndpoint = void 0;
const Endpoint_1 = require("../Endpoint");
const core_1 = require("../../core");
class TestEndpoint extends Endpoint_1.Endpoint {
    constructor() {
        super();
        this._request = {};
        this._test = { name: "Api Test", tests: [] };
    }
    useRequest(request) {
        this._request[request.path] = request.execute;
    }
    addTest(test) {
        this._test.tests.push(test);
    }
    executeTests() {
        return __awaiter(this, void 0, void 0, function* () {
            const time = Date.now();
            core_1.Logger.info("Start the api request tests.");
            return this.executeGroup(this._test).then((result) => {
                let errorCount = 0;
                for (const { error } of result) {
                    if (error) {
                        errorCount++;
                    }
                }
                const during = (Date.now() - time) / 1000;
                core_1.Logger.info(`The api request test end with ${errorCount} error(s) from ${result.length} test(s) in ${during}s.`);
                return result;
            });
        });
    }
    executeGroup(group, store = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            core_1.Logger.info(`Start group test from '${group.name}'`);
            let executeList = Promise.resolve([]);
            if (group.before) {
                executeList = executeList.then((result) => {
                    return group.before(this.core, store).then(() => {
                        return result;
                    });
                });
            }
            if (group.tests) {
                for (const sub of group.tests) {
                    // If it a sub group
                    if (sub.path) {
                        // Execute the single tests
                        executeList = executeList.then((result) => {
                            // Execute request
                            return this.executeRequest(sub, store).then((test) => {
                                const name = sub.name;
                                core_1.Logger.info(`Test at path '${sub.path}' ${name ? `(${name})` : ''}:`
                                    + ` ${test.error ? '\u001b[31mx\u001b[0m' : '\u001b[32m✓\u001b[0m'}`);
                                result.push(test);
                                return result;
                            });
                        });
                    }
                    else {
                        // Copy the store
                        const copyStore = Object.assign({}, store);
                        // Execute the sub group
                        executeList = executeList.then((result) => {
                            return this.executeGroup(sub, copyStore).then((tests) => {
                                // Add the tests results from the sub group
                                return result.concat(tests);
                            });
                        });
                    }
                }
            }
            if (group.after) {
                executeList = executeList.then((result) => {
                    return group.after(this.core, store).then(() => {
                        return result;
                    });
                });
            }
            return executeList.then((result) => {
                core_1.Logger.info(`End group test from  '${group.name}'`);
                return result;
            });
        });
    }
    executeRequest(request, store = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const execute = this._request[request.path];
            if (!execute) {
                return {
                    path: request.path,
                    error: true,
                    errorObject: "Path no found."
                };
            }
            return request.arguments(this.core, store).then((args) => {
                return execute(this.core, args).then((result) => {
                    return request.result(this.core, store, result).then((successfully) => {
                        if (typeof successfully === "boolean") {
                            return {
                                path: request.path,
                                error: !successfully
                            };
                        }
                        else {
                            return {
                                path: request.path,
                                error: true,
                                errorObject: successfully
                            };
                        }
                    });
                }).catch((error) => {
                    return {
                        path: request.path,
                        error: true,
                        errorObject: error
                    };
                });
            });
        });
    }
}
exports.TestEndpoint = TestEndpoint;
//# sourceMappingURL=TestEndpoint.js.map