"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WsClient = void 0;
const core_1 = require("../../core");
class WsClient {
    constructor(core, server, ws) {
        this.BadDataFormat = {
            id: -1,
            connection: [{
                    name: "BadDataFormat",
                    description: "The correct data format is a string with Json format. Every message need an id and a path."
                }],
            data: [],
            error: []
        };
        this.server = server;
        this._ws = ws;
        this.core = core;
        this.initListener();
    }
    close() {
        this._ws.close();
    }
    initListener() {
        const { onMessage, onClose } = this;
        const that = this;
        this._ws.on("message", data => onMessage.bind(that)(data));
        this._ws.on("close", () => onClose.bind(that)());
    }
    onMessage(data) {
        if (typeof data === "string") {
            try {
                this.buildRequest(JSON.parse(data));
                return;
            }
            catch (e) {
            }
        }
        this.send(JSON.stringify(this.BadDataFormat));
    }
    onClose() {
        core_1.Logger.info("WsClient: Close Websocket connection");
        this.server.removeClient(this);
    }
    send(data) {
        this._ws.send(JSON.stringify(data));
    }
    buildRequest(data) {
        if (data && data.id && typeof data.id === "number" && data.path && typeof data.path === "string") {
            const requestObject = {
                id: data.id,
                path: data.path,
                args: {
                    meta: {},
                    data: {}
                }
            };
            if (data.args && typeof data.args === "object") {
                requestObject.args = data.args;
            }
            this.executeRequest(requestObject);
        }
        else {
            this.send(JSON.stringify(this.BadDataFormat));
        }
    }
    executeRequest(data) {
        const func = this.server.endpoints[data.path];
        if (func && typeof func === "function") {
            func(this.core, data.args).then((result) => {
                this.send(Object.assign({
                    id: data.id,
                    connection: []
                }, result));
            });
        }
        else {
            this.send({
                id: data.id,
                connection: [{
                        name: "NoSupportRequest",
                        description: "The path is not supported."
                    }],
                data: [],
                middleware: [],
                validation: []
            });
        }
    }
}
exports.WsClient = WsClient;
//# sourceMappingURL=WsClient.js.map