import { ModelSchema, Optional } from "./schema/ModelSchema";
import { Core } from "../core";
import { ValidatorError } from "./validation/DataValidator";
import { FilterOption, FindFilter } from "./find/FindFilter";
import ExportSchema = Optional.Export.ExportSchema;
export declare class Model {
    readonly name: string;
    readonly schema: ModelSchema;
    private _dependentRefs;
    readonly exec: {
        [key: string]: (...args: any[]) => Promise<any>;
    };
    constructor(name: string, schema: ModelSchema, exec?: {
        [key: string]: (...args: any[]) => Promise<any>;
    });
    private _core;
    get core(): Core;
    init(core: Core): void;
    addRefs(model: Model, name: string, idName: string, depend: boolean): void;
    find(filter: FindFilter, option?: FilterOption): Promise<Value[]>;
    insert(data: Value, protection?: boolean): Promise<ValidatorError[] | Value>;
    delete(filter?: FindFilter): Promise<void>;
    update(data: Value, filter: FindFilter, protection?: boolean): Promise<ValidatorError[]>;
    toExport(data: Value, exportSchema?: ExportSchema): Value;
    valid(data: Value, require?: boolean, protection?: boolean): Promise<ValidatorError[]>;
    private deleteManyByRefs;
    private deleteByRefs;
}
export interface ModelImpl<T> {
    addModel(init: (t: Core) => Model): void;
    getModel(name: string): Model;
    getModels(): Model[];
}
export declare type Value = {
    [index: string]: number | string | boolean | null;
};
