"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildFilter = exports.buildFromDataBase = exports.buildToDataBase = void 0;
function buildToDataBase(model, data, setDefaults = false) {
    const modelSchema = model.schema;
    const modificationData = {};
    if (setDefaults) {
        for (let key in modelSchema) {
            if (data[key] === undefined) {
                const schemaType = modelSchema[key];
                if (schemaType.type === "Boolean" || schemaType.type === "String" || schemaType.type === "Number") {
                    if (schemaType.default) {
                        data[key] = schemaType.default;
                    }
                }
            }
        }
    }
    for (let name in data) {
        let value = data[name];
        const schema = modelSchema[name];
        if (schema) {
            if (schema.name) {
                name = schema.name;
            }
            if (schema.set) {
                value = schema.set(value);
            }
            modificationData[name] = value;
        }
    }
    return modificationData;
}
exports.buildToDataBase = buildToDataBase;
function buildFromDataBase(model, data) {
    const modelSchema = model.schema;
    const modificationData = {};
    for (let name in data) {
        let value = data[name];
        for (const key in modelSchema) {
            const def = modelSchema[key];
            if (def && def.name && def.name === name) {
                name = key;
                break;
            }
        }
        const schema = modelSchema[name];
        if (schema) {
            if (schema.get) {
                value = schema.get(value);
            }
            modificationData[name] = value;
        }
    }
    return modificationData;
}
exports.buildFromDataBase = buildFromDataBase;
function buildFilter(model, filter) {
    const modelSchema = model.schema;
    const modificationFilter = {};
    for (let name in modelSchema) {
        const schema = modelSchema[name];
        const value = filter[name];
        if (!value) {
            continue;
        }
        if (schema.name) {
            name = schema.name;
        }
        switch (schema.type) {
            case "Id":
                if (typeof value === "string" || Array.isArray(value)) {
                    modificationFilter[name] = {
                        type: "Id",
                        value
                    };
                }
                break;
            case "Refs":
                if (typeof value === "string") {
                    modificationFilter[name] = {
                        type: "Refs",
                        value
                    };
                }
                break;
            case "String":
                if (typeof value === "string") {
                    modificationFilter[name] = {
                        type: "String",
                        value
                    };
                }
                break;
            case "Boolean":
                if (typeof value === "boolean") {
                    modificationFilter[name] = {
                        type: "Boolean",
                        value
                    };
                }
                break;
            case "Number":
                if (typeof value === "object") {
                    modificationFilter[name] = Object.assign({ type: "Number" }, value);
                }
                else if (typeof value === "number") {
                    modificationFilter[name] = {
                        type: "Number",
                        value
                    };
                }
                break;
        }
    }
    return modificationFilter;
}
exports.buildFilter = buildFilter;
//# sourceMappingURL=Modification.js.map