"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validWithModel = void 0;
function validWithModel(data, model, require = false, protection = true) {
    return __awaiter(this, void 0, void 0, function* () {
        const promises = [];
        for (const name in data) {
            promises.push(Promise.resolve(() => {
                const value = data[name];
                const schema = model.schema[name];
                const error = { name };
                if (!schema) {
                    error.notDefine = `The '${name}' is not define.`;
                    return error;
                }
                if ((schema.require && value === null) || (require && schema.require && !value)) {
                    error.require = `The '${name}' must be a value.`;
                    return error;
                }
                if (protection) {
                    if (schema.protection) {
                        error.require = `The '${name}' is write protection.`;
                        return error;
                    }
                }
                switch (schema.type) {
                    case "String":
                        if (typeof value === "string") {
                            if (schema.min && value.length < schema.min) {
                                error.min = `The length '${value}' of string is to low. The must be ${schema.min} or over.`;
                            }
                            if (schema.max && schema.max < value.length) {
                                error.max = `The length '${value}' of string is to height. The must be ${schema.max} or lower.`;
                            }
                            if (schema.validate && schema.validate(value)) {
                                error.validate = `'${value}' is not valid.`;
                            }
                        }
                        else {
                            error.type = "The type must be a string.";
                        }
                        return error;
                    case "Boolean":
                        if (typeof value !== "boolean") {
                            error.type = "The type must be a string.";
                        }
                        return error;
                    case "Number":
                        if (typeof value === "number") {
                            if (schema.min && value < schema.min) {
                                error.min = `${value} is to low. The must be ${schema.min} or over.`;
                            }
                            if (schema.max && schema.max < value) {
                                error.max = `${value} is to height. The must be ${schema.max} or lower.`;
                            }
                            if (schema.validate && schema.validate(value)) {
                                error.validate = `${value} is not valid.`;
                            }
                        }
                        else {
                            error.type = "The type must be a number.";
                        }
                        return error;
                    case "Refs":
                        if (typeof value === "string") {
                            const refModel = model.core.getModel(schema.model);
                            if (refModel) {
                                const idFilter = {};
                                idFilter[schema.idName] = value;
                                return refModel.find(idFilter).then((found) => {
                                    if (!found) {
                                        error.dependent = "The reference is not correct.";
                                    }
                                    return error;
                                });
                            }
                        }
                        error.type = "The type must be a string of id.";
                        return error;
                    case "Id":
                        error.type = "The type id can not change.";
                        return error;
                }
                error.notDefine = "The type of schema is not define.";
                return error;
            }));
        }
        return Promise.all(promises).then((results) => {
            const errors = [];
            for (const result of results) {
                if (Object.keys(result).length > 1) {
                    errors.push(result);
                }
            }
            return errors;
        }).then((errors) => {
            for (const name in model.schema) {
                const mustType = model.schema[name];
                if (!((mustType.type === "Boolean" || mustType.type === "String" || mustType.type === "Number") && mustType.default)) {
                    if (mustType.require && !data[name]) {
                        errors.push({
                            name,
                            require: "The type must be require"
                        });
                    }
                }
            }
            return errors;
        });
    });
}
exports.validWithModel = validWithModel;
//# sourceMappingURL=Validator.js.map