"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Service = void 0;
const core_1 = require("../core");
class Service {
    constructor(name, nextTime, interval = false) {
        this._caller = null;
        this._name = name;
        this._nextTime = nextTime;
        this._interval = interval;
    }
    init(core) {
        return __awaiter(this, void 0, void 0, function* () {
            this._core = core;
            core_1.Logger.info(`Init Service "${this._name}"`);
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            core_1.Logger.info(`Start Service "${this._name}"`);
            if (this._caller !== null) {
                core_1.Logger.error(`Event can't start in Service "${this._name}"`);
                return;
            }
            const that = this;
            const call = () => {
                that.call.call(that);
            };
            if (this._interval) {
                this._caller = setInterval(call, this._nextTime);
                this.call();
            }
            else {
                this._caller = setTimeout(call, this._nextTime, this);
            }
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            core_1.Logger.info(`Stop Service "${this._name}"`);
            if (this._caller === null) {
                core_1.Logger.error(`Event can't stop in Service "${this._name}"`);
                return;
            }
            if (this._interval) {
                clearInterval(this._caller);
            }
            else {
                clearTimeout(this._caller);
            }
            this._caller = null;
        });
    }
    call() {
        core_1.Logger.info(`Call Service "${this._name}"`);
        this.onEvent(this._core);
        if (this._interval === null) {
            this._caller = null;
        }
    }
}
exports.Service = Service;
//# sourceMappingURL=Service.js.map