"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Trigger = void 0;
const core_1 = require("../core");
class Trigger {
    constructor() {
        this._listener = [];
    }
    addListener(listener) {
        this._listener.push(listener);
        core_1.Logger.info("Add trigger listener");
    }
    triggerInsert(model, insert) {
        return __awaiter(this, void 0, void 0, function* () {
            const triggers = [];
            for (const trigger of this._listener) {
                if (trigger.insert) {
                    triggers.push(trigger.insert(model, insert));
                }
            }
            return Promise.all(triggers).then();
        });
    }
    triggerUpdate(model, data, updates) {
        return __awaiter(this, void 0, void 0, function* () {
            const triggers = [];
            for (const trigger of this._listener) {
                if (trigger.update) {
                    triggers.push(trigger.update(model, data, updates));
                }
            }
            return Promise.all(triggers).then();
        });
    }
    triggerDelete(model, data) {
        return __awaiter(this, void 0, void 0, function* () {
            const triggers = [];
            for (const trigger of this._listener) {
                if (trigger.delete) {
                    triggers.push(trigger.delete(model, data));
                }
            }
            return Promise.all(triggers).then();
        });
    }
    clear() {
        this._listener = [];
        core_1.Logger.info("Clear Trigger");
    }
}
exports.Trigger = Trigger;
//# sourceMappingURL=Trigger.js.map