import {Core, Endpoint, generateList, group, param, request, TestEndpoint, TriggerLogger, VirtualDB} from "../../src";

import Company from "./model/Company";
import User from "./model/User";
import ApiTest from "./test/ApiTest";
import {RequestCounter} from "./middleware/RequestCounter";
import MDGenerator from "../../src/endpoint/generator/descriptor/MDGenerator";


const mongoDB = new VirtualDB();

const core: Core = new Core(mongoDB);

// Init trigger
core.addTrigger(TriggerLogger);

// Init model
core.addModel(Company);
core.addModel(User);

// Init Endpoints
const endpoint: Endpoint = new TestEndpoint();

const pathOption = {
    core,
    paths: [{
        path: 'user',
        middleware: new RequestCounter(),
        descriptor: [
            group("User","User Option"),
            request("Get User","Return all user form the project"),
            param('userId', 'string', true, false)
        ],
        execute: generateList("User", {id: "userId"})
    }]
}

endpoint.generateApiRequest(pathOption);

console.log(MDGenerator("Zodora", "www.zodora.com/api", pathOption));

core.addEndpoint("RestAPI", endpoint);

core.createAndStart().then(() => {


})
